﻿using System.Drawing.Drawing2D;
using Excavator.Entities;


namespace Excavator.Drawnings;
/// <summary>
/// Класс, отвечающий за прорисовку и перемещение базового объекта сущности
/// </summary>
public class DrawningTrackedVehicle
{
    
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityTrackedVehicle? EntityTrackedVehicle { get; protected set; }

    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;

    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;

    /// <summary>
    /// Левая координата прорисовки эскаватора
    /// </summary>
    protected int? _startPosX;

    /// <summary>
    /// Верхняя кооридната прорисовки экскаватора
    /// </summary>
    protected int? _startPosY;

    /// <summary>
    /// Ширина прорисовки экскаватора
    /// </summary>
    private readonly int _drawningExcavatorWidth = 110;

    /// <summary>
    /// Высота прорисовки экскаватора
    /// </summary>
    private readonly int _drawningExcavatorHeight = 126;


    /// <summary>
    /// Координата X объекта
    /// </summary>
    public int? GetPosX => _startPosX;

    /// <summary>
    /// Координата Y объекта
    /// </summary>
    public int? GetPosY => _startPosY;

    /// <summary>
    /// Ширина объекта
    /// </summary>
    public int GetWidth => _drawningExcavatorWidth;

    /// <summary>
    /// Высота объекта
    /// </summary>
    public int GetHeight => _drawningExcavatorHeight;
    /// <summary>
    /// Пустой конструктор
    /// </summary>
    private DrawningTrackedVehicle()
    {
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }

    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
 


    public DrawningTrackedVehicle(int speed, double weight, Color bodyColor) : this()
    {
        EntityTrackedVehicle = new EntityTrackedVehicle(speed, weight, bodyColor);
       
    }

    /// <summary>
    /// Конструктор для наследников
    /// </summary>
    /// <param name="drawningExcavatorWidth">Ширина прорисовки гусеничной машины</param>
    /// <param name="drawningExcavatorHeight">Высота прорисовки гусеничной машины</param>




    protected DrawningTrackedVehicle(int drawningExcavatorWidth, int drawningExcavatorHeight) : this()
    {
        _drawningExcavatorWidth = drawningExcavatorWidth;
        _pictureHeight = drawningExcavatorHeight;
      
    }



    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>


    public bool SetPictureSize(int width, int height)
    {
        if (width > _drawningExcavatorWidth && height > _drawningExcavatorHeight)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            if (_startPosX != null && _startPosY != null)
            {
                if (_startPosX.Value < 0)
                {
                    _startPosX = 0;
                }
                if (_startPosY.Value < 0)
                {
                    _startPosY = 0;
                }
                if (_startPosX.Value + _drawningExcavatorWidth > _pictureWidth)
                {
                    _startPosX = _pictureWidth - _drawningExcavatorWidth;
                }
                if (_startPosY.Value + _drawningExcavatorHeight > _pictureHeight)
                {
                    _startPosY = _pictureHeight - _drawningExcavatorHeight;
                }
            }

            return true;
        }
        return false;

    }
    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }
        else
        {
            _startPosX = x;
            _startPosY = y;
            if (_startPosX.Value < 0)
            {
                _startPosX = 0;
            }
            if (_startPosY.Value < 0)
            {
                _startPosY = 0;
            }
            if (_startPosX + _drawningExcavatorWidth > _pictureWidth)
            {
                _startPosX = _pictureWidth - _drawningExcavatorWidth;
            }
            if (_startPosY + _drawningExcavatorHeight > _pictureHeight)
            {
                _startPosY = _pictureHeight - _drawningExcavatorHeight;
            }
        }

    }

    /// <summary>
    /// Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещене выполнено, false - перемещение невозможно</returns>
    public bool MoveTransport(DirectionType direction)
    {
        if (EntityTrackedVehicle == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return false;
        }

        switch (direction)
        {
            //влево
            case DirectionType.Left:

                if (_startPosX.Value - EntityTrackedVehicle.Step > 0)
                {
                    _startPosX -= (int)EntityTrackedVehicle.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityTrackedVehicle.Step > 0)
                {
                    _startPosY -= (int)EntityTrackedVehicle.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:

                if (_startPosX.Value + _drawningExcavatorWidth + EntityTrackedVehicle.Step < _pictureWidth)
                {

                    _startPosX += (int)EntityTrackedVehicle.Step;

                }
                return true;
            //вниз
            case DirectionType.Down:
                if (_startPosY.Value + _drawningExcavatorHeight + EntityTrackedVehicle.Step < _pictureHeight)
                {
                    _startPosY += (int)EntityTrackedVehicle.Step;
                }
                return true;
            default:
                return false;
        }
    }




    public static GraphicsPath RoundedRect(Graphics g, Rectangle bounds, int radius)
    {
        int diameter = radius * 2;
        Size size = new Size(diameter, diameter);
        Rectangle arc = new Rectangle(bounds.Location, size);
        GraphicsPath path = new GraphicsPath();

        if (radius == 0)
        {
            path.AddRectangle(bounds);
            return path;
        }

        path.AddArc(arc, 180, 90); 
        arc.X = bounds.Right - diameter;
        path.AddArc(arc, 270, 90);
        arc.Y = bounds.Bottom - diameter;
        path.AddArc(arc, 0, 90);
        arc.X = bounds.Left;
        path.AddArc(arc, 90, 90);

        g.FillPath(Brushes.Black, path);

        path.CloseFigure();
        return path;
    }


    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    /// 

    public virtual void DrawTransport(Graphics g)
    {
        if (EntityTrackedVehicle == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }

        Pen pen = new(Color.Black);
 
        Brush brBlue = new SolidBrush(Color.LightBlue);
        Brush brGray = new SolidBrush(Color.Gray);
        Brush brRed = new SolidBrush(Color.Red);
        Brush brYellow = new SolidBrush(Color.Yellow);
        Brush brBlack = new SolidBrush(Color.Black);
        Brush bodycolor = new SolidBrush(EntityTrackedVehicle.BodyColor);  


        int bodyHeight = 90;
        int cabineHeight = 40;
        int pipeHeight = 35;
        int bucketWidth = 10;
        int pipeWidth = 7;

        int pipeOffsetX = 25;
        int cabineOffsetX = 50;

        int wheelsOffsetX = 20;
        int wheelsHeight = 40;

        //кузов
        g.DrawRectangle(pen, _startPosX.Value + bucketWidth, _startPosY.Value + cabineHeight, bodyHeight, bodyHeight - cabineHeight);
        g.FillRectangle(bodycolor, _startPosX.Value + bucketWidth, _startPosY.Value + cabineHeight, bodyHeight, bodyHeight - cabineHeight);

        //труба
        g.DrawRectangle(pen, _startPosX.Value + bucketWidth + pipeOffsetX, _startPosY.Value + cabineHeight - pipeHeight, pipeWidth, pipeHeight);
        g.FillRectangle(brBlack, _startPosX.Value + bucketWidth + pipeOffsetX, _startPosY.Value + cabineHeight - pipeHeight, pipeWidth, pipeHeight);

        //кабина
        g.DrawRectangle(pen, _startPosX.Value + bucketWidth + cabineOffsetX, _startPosY.Value, cabineHeight, cabineHeight);
        g.FillRectangle(brBlue, _startPosX.Value + bucketWidth + cabineOffsetX, _startPosY.Value, cabineHeight, cabineHeight);

        //трак
        g.DrawPath(pen, RoundedRect(g, new Rectangle(_startPosX.Value , _startPosY.Value + bodyHeight, wheelsOffsetX + bodyHeight, wheelsHeight), 15));

        Pen bPen = new(Color.Black);
        bPen.Width = 3;

        Rectangle wheel1 = new Rectangle(
            _startPosX.Value + bucketWidth + wheelsOffsetX - 25,
            _startPosY.Value + bodyHeight + wheelsHeight / 4,
            25, 25);

        Rectangle wheel2 = new Rectangle(
            _startPosX.Value + bucketWidth + (bodyHeight - wheelsOffsetX),
            _startPosY.Value + bodyHeight + wheelsHeight / 4,
            25, 25);

        g.DrawEllipse(bPen, wheel1);
        g.DrawEllipse(bPen, wheel2);

        g.FillEllipse(brBlue, wheel1);
        g.FillEllipse(brBlue, wheel2);

        //колеса
        g.DrawEllipse(bPen, _startPosX.Value + bucketWidth + wheelsOffsetX + 5, _startPosY.Value + bodyHeight + wheelsHeight / 2, 13, 13);
        g.DrawEllipse(bPen, _startPosX.Value + bucketWidth + wheelsOffsetX + 20, _startPosY.Value + bodyHeight + wheelsHeight / 2, 13, 13);
        g.DrawEllipse(bPen, _startPosX.Value + bucketWidth + wheelsOffsetX + 35, _startPosY.Value + bodyHeight + wheelsHeight / 2, 13, 13);
        g.FillEllipse(brRed, _startPosX.Value + bucketWidth + wheelsOffsetX + 5, _startPosY.Value + bodyHeight + wheelsHeight / 2, 13, 13);
        g.FillEllipse(brRed, _startPosX.Value + bucketWidth + wheelsOffsetX + 20, _startPosY.Value + bodyHeight + wheelsHeight / 2, 13, 13);
        g.FillEllipse(brRed, _startPosX.Value + bucketWidth + wheelsOffsetX + 35, _startPosY.Value + bodyHeight + wheelsHeight / 2, 13, 13);

        g.DrawEllipse(bPen, _startPosX.Value + bucketWidth + wheelsOffsetX + 13, _startPosY.Value + bodyHeight + wheelsHeight / 6, 9, 9);
        g.DrawEllipse(bPen, _startPosX.Value + bucketWidth + wheelsOffsetX + 28, _startPosY.Value + bodyHeight + wheelsHeight / 6, 9, 9);
        g.FillEllipse(brYellow, _startPosX.Value + bucketWidth + wheelsOffsetX + 13, _startPosY.Value + bodyHeight + wheelsHeight / 6, 9, 9);
        g.FillEllipse(brYellow, _startPosX.Value + bucketWidth + wheelsOffsetX + 28, _startPosY.Value + bodyHeight + wheelsHeight / 6, 9, 9);       
    }
}



